#ifndef rep_decoder_h
#define rep_decoder_h


#ifndef WIN32
#include <sys/param.h>
#include <sys/uio.h>
#endif
#include "sys-time.h"
#include "inet.h"
#include "rv.h"
#include "tclcl.h"
#include "source.h"
#include "ntp-time.h"
#include "vidreps.h"

class RepDecoder : public PacketHandler {
protected:
  RepDecoder(int hdrlen);

public:
  virtual ~RepDecoder();

  static double gettimeofday();

  virtual int command(int argc, const char*const* argv);

  virtual void recv(pktbuf *pb);
  virtual void post_delay_recv(pktbuf *pb) = 0;
  virtual void set_frame_buffer(VidRep *fb) = 0;
  virtual void resize(int inw, int inh) {
    inw_ = inw;
    inh_ = inh;
  };

protected:
  int inw_;
  int inh_;

  char *callback_;

  int delay_flag_;
  
  class PktBufDelayQueue {
  public:
    PktBufDelayQueue() : head_(0), tail_(0) {};
    inline void insert (pktbuf *pb) {
      if (tail_ == 0) {
	head_ = tail_ = pb;
	tail_->next = 0;
      } else {
	tail_->next = pb;
	tail_ = tail_->next;
	tail_->next = 0;
      }
    };
    inline pktbuf *get_next () {
      pktbuf *ret;
      if (head_ == 0) 
	return 0;
      ret = head_;
      head_ = head_->next;
      if (head_ == 0) {
	tail_ = 0;
      }
      ret->next = 0;
      return ret;
    }
  protected:
    pktbuf *head_;
    pktbuf *tail_;
  } delay_q_;  
};

#endif
