#
# Copyright (c) 1996 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the MASH Research
#       Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/avswitch-client/avswitch-ui.tcl,v 1.10 1998/12/24 04:02:16 hodes Exp $
#




#
# UI for talking to an AVSwitchServer
#
Class AVSwitchUI 


#
# init method takes window and and address specificationx
#
AVSwitchUI public init {w addrspec} {

    $self instvar al_ audio_ video_ routingType_ in_update_

    # if addrspec includes a name, turn it into IP addr
    set firstchar [string index $addrspec 0]
    if [string match \[a-zA-Z\] $firstchar] {
	set n [lindex [split $addrspec "/"] 0]
	set p [lindex [split $addrspec "/"] 1]
        set s [gethostbyname $n]
        if { $s == "" } {
            puts "cannot find address for '$n'"
            exit
        }
        set addrspec $s/$p
    }
    
    set routingType_ b
    set al_ [new ALM/AVSw/Client $addrspec $self]
    for {set j 1} {$j < 9} {incr j} {
	set audio_($j) 0
	set video_($j) 0
    }
    set in_update_ 0
    
    $al_ announce refresh_settings
    $self build_gui $w
    update
    $self refresh_ui
}

#
# destructor method
#
AVSwitchUI public destroy {} {
    $self instvar al_
    delete $al_
    eval [list $self] next
}

AVSwitchUI public print_status {} {
    $self instvar audio_ video_

    puts ""
    puts "       Routing"
    puts "   1:2:3:4:5:6:7:8"
    for {set i 1} {$i <= 8} {incr i} {
	puts -nonewline "$i: "
	if {$audio_($i) == $video_($i)} {
	    # just a "B"
	    $self put_letter "B" $audio_($i)
	    puts ""
	} else {
	    # put a "V" and "A"
	    if {$audio_($i) < $video_($i)} {
		$self put_letter "A" $audio_($i)
		$self put_letter "V" [expr $video_($i) - $audio_($i)] 
		puts ""
	    } else {
		$self put_letter "V" $video_($i)
		$self put_letter "A" [expr $audio_($i) - $video_($i)]
		puts ""
	    }
	}
    }
    puts ""
}

AVSwitchUI private put_letter {letter dist} {
    for {set j 1} {$j < $dist} {incr j} {
	puts -nonewline "  "
    }
    puts -nonewline "$letter "
}

AVSwitchUI private build_gui {w} {
    $self instvar al_ routingType_ matrix_ input_labels_
    
    # build bottom informational label
    set il $w.infolabel
    frame $il -relief groove 
    label $il.l -text "A/V Switch Controller (326 Soda CoLab)"
    pack $il -side bottom -fill x -expand 1
    pack $il.l -in $il -side bottom
    
    # build button colums
    set b $w.btns
    frame $b -relief groove
    pack $b -side left -anchor s
    button $b.b1 -command "$al_ announce refresh_settings" \
	    -text "resynchronize"
    button $b.b2 -command "$al_ announce test_LEDs" -text "test lights"
    button $b.b3 -command "$self print_status" -text "print status"
    button $b.e -command "exit" -text "EXIT"
    pack $b.e $b.b3 $b.b2 $b.b1 -fill x -side bottom
    
    frame $b.routing -relief groove
    label $b.routing.label -text "Route:"
    radiobutton $b.routing.a -variable routingType -text "Audio" -value a \
	    -fg darkgreen -command "$self set routingType_ a"
    radiobutton $b.routing.v -variable routingType -text "Video" -value v \
	    -fg blue -command "$self set routingType_ v"
    radiobutton $b.routing.b -variable routingType -text "Both" -value b \
	    -fg "#b03060" -command "$self set routingType_ b"
    pack $b.routing.label -in $b.routing -anchor w -side left
    pack $b.routing.b $b.routing.v $b.routing.a -in $b.routing
    pack $b.routing -fill x -expand 1 -pady 2 -side bottom
    $b.routing.$routingType_ invoke
    
    # build routing matrix
    set m $w.matrix
    set matrix_ $m
    frame $m
    pack $m -side right
    set rowLabels [list "monitor 1" "monitor 2" "monitor 3" "VCR" \
	    "(null)" "diamond" "heart" "club"]
    
    for {set i 1} {$i <= 8} {incr i} {
        frame $m.rowFrame$i
	label $m.rowFrame$i.l -text [lindex $rowLabels [expr $i-1]]
    }
    
    for {set i 1} {$i <= 8} {incr i} {
        for {set j 1} {$j <= 8} {incr j} {
            checkbutton $m.r$i$j -command "$self invoke_button $i $j"
	}
    }
    # these are for the iput labels
    $self build_input_labels
    frame $m.rowFrame0
    pack $m.rowFrame0 -expand 1 -in $m -anchor w
    label $m.rowFrame0.l -image $input_labels_
    pack $m.rowFrame0.l -in $m.rowFrame0 -side left -padx 3
    for {set i 1} {$i <= 8} {incr i} {
        pack $m.rowFrame$i -expand 1 -in $m -anchor w
	pack $m.rowFrame$i.l -in $m.rowFrame$i -side right
    }
    for {set i 1} {$i <= 8} {incr i} {
        for {set j 1} {$j <= 8} {incr j} {
            pack $m.r$i$j -in $m.rowFrame$i -side left
        }
    }
    
    bind . <q> exit
}

AVSwitchUI private repaint_matrix {} {
    $self instvar audio_ video_ matrix_ in_update_

    if {$in_update_ == 1} {
	return
    }
    
    set m $matrix_
    
    # check if still waiting for update
    if {[lsearch [array get audio_] 0] != -1 || \
	    [lsearch [array get video_] 0] != -1} {
#puts "waiting for update..."
	[$self set al_] announce refresh_settings
	return
    }

    set in_update_ 1

    for {set i 1} {$i <= 8} {incr i} {
        for {set j 1} {$j <= 8} {incr j} {
            $m.r$i$j deselect
	}
    }
    for {set i 1} {$i <= 8} {incr i} {
	if {$audio_($i) == $video_($i)} {
	    $m.r$i$audio_($i) configure -selectcolor "#b03060"
	    $m.r$i$audio_($i) select
	} else {
	    $m.r$i$audio_($i) configure -selectcolor darkgreen
	    $m.r$i$audio_($i) select
	    $m.r$i$video_($i) configure -selectcolor blue
	    $m.r$i$video_($i) select			
	}
    }
    set in_update_ 0
}

AVSwitchUI private invoke_button {i j} {
    $self instvar routingType_ audio_ video_
    #puts "$routingType_: $i $j"

    set al [$self set al_]
    if {$routingType_ == "a"} {
	set audio_($i) $j
	$al announce "route_audio $i $j"
    } elseif {$routingType_ == "v"} {
	set video_($i) $j
	$al announce "route_video $i $j"
    } else {
	set audio_($i) $j
	set video_($i) $j
	$al announce "route_both $i $j"
    }
    $self repaint_matrix
}

AVSwitchUI private refresh_ui {} {
    $self repaint_matrix
}


#
# This is an embedded gif of vertical input labels
#

# -- These cmds generate a list that can then be rotated, saved
# -- as a .gif, and put into the avswitch via conversion by image2tcl
#
#set columnLabels [list "heart" "diamond" "club" "VCR" "cam1 (red)" \
#		"cam2 (yellow)" "cam3 (blue)" "cam4 (green)"]
#for  {set i 1} {$i <= 8} {incr i} {
#	label .l$i -text [lindex $columnLabels [expr $i-1]]
#	pack .l$i -anchor w -pady  3 }
#
# -- The result for the current inputs are embedded in the following
AVSwitchUI private build_input_labels {} {
    $self instvar input_labels_

    set imageObject__ [new Image/GIF "\107\111\106\70\67\141\302\134\43\126\134\43\360\134\43\134\43\134\43\134\43\134\43\330\330\330\54\134\43\134\43\134\43\134\43\302\134\43\126\134\43\134\43\2\376\214\217\251\313\355\17\243\234\21\330\113\263\336\274\373\17\52\327\210\205\346\211\246\152\126\42\355\12\307\362\54\1\216\115\347\372\316\367\376\17\14\12\207\304\220\245\150\40\341\220\114\30\111\250\34\65\247\252\247\357\225\74\122\267\47\155\157\271\134\43\163\307\23\53\371\214\276\111\323\354\166\130\374\65\273\347\11\171\334\116\167\343\277\371\276\10\276\241\304\1\350\127\30\30\345\125\43\150\250\267\106\201\25\134\43\251\266\307\310\25\305\162\123\251\371\40\106\250\203\341\271\331\105\311\131\27\44\225\50\212\202\230\12\161\171\150\23\232\31\231\244\132\325\52\71\331\252\170\264\353\132\113\153\273\72\313\203\143\14\353\50\314\304\32\10\54\333\100\252\174\212\210\234\53\15\304\74\155\355\332\173\155\222\135\106\215\15\372\354\135\166\120\136\47\316\262\210\31\234\156\136\352\242\235\174\376\36\17\362\252\1\152\64\330\216\337\41\232\256\110\360\210\261\333\6\160\307\261\140\7\353\51\112\70\52\326\40\147\325\12\272\260\10\221\1\52\376\214\4\5\316\360\225\321\336\302\175\16\271\351\173\364\57\144\15\212\127\326\241\364\250\122\35\314\77\370\300\371\231\371\7\44\57\235\272\160\232\114\111\207\147\70\204\32\201\126\260\331\17\130\41\241\57\231\316\173\272\17\235\324\56\246\14\71\345\124\342\252\122\216\14\131\176\63\332\310\47\72\54\351\220\356\44\227\102\354\226\223\341\74\230\75\253\64\246\134\134\31\140\323\204\52\153\60\157\325\137\320\126\325\105\3\211\150\26\70\202\73\346\344\165\230\352\46\126\5\31\67\375\233\223\155\322\305\152\7\27\216\254\365\142\311\174\220\347\112\223\354\71\264\350\170\225\107\57\355\154\172\361\336\324\105\236\161\355\344\71\25\306\136\227\307\262\205\115\262\64\345\104\26\337\366\164\344\373\47\127\44\301\65\146\231\52\22\252\275\270\35\165\357\106\15\125\113\357\270\34\245\207\371\272\231\315\360\261\266\173\167\32\56\221\346\7\320\155\266\43\67\354\356\374\241\67\371\130\7\65\157\76\342\140\177\331\261\326\377\115\237\54\164\314\257\376\357\277\41\157\237\177\377\11\310\137\165\51\1\370\233\201\142\55\221\331\152\1\115\346\126\172\217\100\330\125\5\352\105\325\136\206\343\11\127\335\67\27\362\125\141\105\371\65\150\31\211\315\311\246\333\176\45\22\330\135\145\10\102\243\142\213\44\336\346\325\143\231\25\67\240\213\24\116\244\241\161\367\364\230\107\174\41\66\3\44\115\54\312\124\233\214\263\71\150\343\210\375\35\251\131\140\307\211\130\221\167\352\124\211\145\156\120\256\50\244\10\65\362\70\241\227\105\22\271\141\221\374\20\71\122\204\141\62\171\326\223\111\312\350\44\222\4\255\267\345\211\67\176\107\245\226\46\306\10\147\125\151\206\271\247\213\174\262\324\341\217\356\374\271\222\241\204\132\125\347\242\30\42\152\241\242\311\125\370\242\227\225\42\351\334\170\261\274\71\147\235\264\315\270\51\56\133\206\12\45\216\342\45\145\342\73\340\115\211\146\227\226\64\52\127\214\246\372\70\344\121\234\42\267\52\163\153\346\32\342\245\121\12\312\235\133\240\6\313\16\261\50\31\333\71\26\253\272\202\371\32\233\57\221\271\54\210\321\312\363\241\264\275\2\131\150\265\1\362\72\155\136\331\166\333\127\164\56\15\345\251\254\260\162\231\145\103\234\352\63\153\216\1\215\352\152\212\260\312\373\102\1\134\43\73"]
	
    image create photo AVSwInputs -data $imageObject__
    set input_labels_ AVSwInputs
}


#----------------------------------------------

import AnnounceListenManager

#
# AVSwitch RPC-style interface client
#
Class ALM/AVSw/Client -superclass AnnounceListenManager


ALM/AVSw/Client public init {addrspec parent {mtu 1500}} {
    eval [list $self] next $addrspec $mtu

    $self instvar parent_
    set parent_ $parent
    $self ttl 16
}

#
# receive (only) matrix state updates of the form 
# "v: <array> ; a: <array>"
#
ALM/AVSw/Client private recv_announcement {addr port data size} {
    #puts "Msg: $addr/$port\[$size\]: $data"
    $self instvar parent_

    # filter other client's requests
    if {$data == "refresh_settings"} {return}

    set t [split $data ";"]
    set vid [lindex $t 0]
    set aud [lindex $t 1]
    set vid [string trim  $vid " "]
    set aud [string trim  $aud " "]
    if {[string range $vid 0 2] != "v: " || [string range $aud 0 2] != "a: "} {
		puts "bad msg received from AVSw server: $data"
		return
    }
    #puts "got status"
    set vid [string range $vid 3 end]
    set aud [string range $aud 3 end]
    $parent_ array set audio_ $aud
    $parent_ array set video_ $vid
    $parent_ refresh_ui
} 


