#
# Copyright (c) 1996 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the MASH Research
#       Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/powerswitch-server/powersw-srv.tcl,v 1.5 1998/12/30 18:05:32 hodes Exp $
#


import SerialChannel/Powerswitch-Microgreen-Duet UDPServer Rendezvous

# PowerSwitch RPC-style interface
#
Class UDPServer/PowerSw -superclass UDPServer

#
UDPServer/PowerSw public init {addrspec parent {mtu 1500}} {
    $self instvar parent_ pending_announce_
    set parent_ $parent
    set pending_announce_ -1

    eval [list $self] next $addrspec $mtu
}

#
UDPServer/PowerSw private recv {addr port data size} {
    puts "Msg: $addr/$port \[$size\]: $data"

    $self instvar parent_ pending_announce_

    set theMethod [lindex $data 0]
    switch $theMethod {
	set_power {$parent_ set_power "$data"}
	default {puts "bad msg: no action."}
    }
    if {$pending_announce_ != -1} {
	after cancel $pending_announce_
    }
    set pending_announce_ [after 500 $self announce_reply]
}

#
UDPServer/PowerSw private announce_reply {} {
    $self instvar parent_ pending_announce_
    
    set p [$parent_ set powerswitch_]
    set state [$p array get plugState_]
    puts "$state"
    
    $self announce "$state"
    set pending_announce_ -1
}


#
# a UI-less server that accepts RPC-like commands and
# returns results for controlling a powerswitch
#
Class PowerSwitchServer

#
PowerSwitchServer public init {addrspec} {

    $self instvar powerswitch_ al_

    set powerswitch_ [new SerialChannel/Powerswitch-Microgreen-Duet 1024]
    set al_ [new UDPServer/PowerSw $addrspec $self]

    set raddr [$self get_option rendez]
    if {$raddr != ""} {
	$self start_rendez $raddr $addrspec
    }

}

#
PowerSwitchServer public start_rendez {raddr ctrladdr} {
    $self instvar rv_
    set rv_ [new Rendezvous $raddr]
    set m "will-provide: mash-object=[$self info class] ctrlspec=$ctrladdr"
    set uid [$self get_option uniqid]
    if {$uid != ""} {set m "$m uniqid=$uid"}
    puts "starting rv msgs on $raddr"
    $rv_ start $m
}

#
PowerSwitchServer public destroy {} {
	$self instvar al_ powerswitch_

	delete $al_
	delete $avswitch_

	eval [list $self] next
}

#
PowerSwitchServer private set_power {data} {
	$self instvar powerswitch_

	# 'set_power number onOff'
	if {[llength $data] != 3} {
		puts "bad params to set_power: $data"
		return
	}

	set num [lindex $data 1]
	set onOff [lindex $data 2]

	$powerswitch_ set_power $num $onOff
	$powerswitch_ set plugState_($num) $onOff
}
