import Prefops/Binary

#
# Prefops for Newscast appplication.
# Receiver reports include categories and associated keywords
# lpv: {{category1 {kw1 kw2 ...}} {category2 {kw1 kw2 ...}} ...}
#
Class Prefops/Newscast -superclass Prefops/Binary

#
# Find the cost of the current partition.
# It is the percentage of extra cat,kw pair than interested, on average.
#
Prefops/Newscast public get_cost { alg } {
	$self instvar pref_

	array set cluts [$alg get_cluts]
	set num_cluts [$alg set num_cluts_]
	set pc 0
	foreach recv_id [array names pref_] {
		set ex 0
		set total 0
		for {set i 1} {$i <= $num_cluts} {incr i} {
			set flag 0
			set tmp 0
			if [info exists cluts(src,$i)] {
				foreach ck $cluts(src,$i) {
					if ![$self is_interested $recv_id $ck] {
						incr tmp
					} else {
						set flag 1
					}
				}
				if { $flag == 1 } {
					set ex [expr $ex + $tmp]
					set total [expr $total + [llength $cluts(src,$i)]]
				}
			}
		}
		set pc [expr double($ex) / double($total)]
	}
	set pc [expr double($pc) / double([llength [array names pref_]])]
	puts stderr "avg percent of extra symbols: $pc"
	return $pc	
}

Prefops/Newscast public get_clutcost { lck } {
	set pv [$self get_zero_src_pv]
	foreach ck $lck {
		set pv [$self add $pv [$self get_src_pv $ck]]
	}
	set pv [$self scale $pv]
	set cost 0
	foreach v $pv {
		set idx [lindex $v 0]
		set val [lindex $v 1]
		set cost [expr $sum + (1 - $val)]
	}
	set cost [expr double($cost) / double([llength $pv])]
	return $cost
}

#
# Return whether receiver recv_id is interested in cat,kw pair ck.
#
Prefops/Newscast public is_interested { recv_id ck } {
	$self instvar pref_

	foreach p $pref_($recv_id) {
		set cat [lindex $p 0]
		set lkw [lindex $p 1]
		if { "$cat" == "[lindex $ck 0]" && \
			( [llength [lintersect $lkw [lindex $ck 1]]] > 0 || \
			"$lkw" == "*" ) } {
			return 1
		}
	}
	return 0
}

#
# return a list of keywords specified by receiver recv_id in the category cat
#
Prefops/Newscast private get_kw { recv_id cat } {
	$self instvar pref_

	if [info exists pref_($recv_id)] {
		foreach p $pref_($recv_id) {
			set c [lindex $p 0]
			set lkw [lindex $p 1]
			if { "$cat" == "$c" } {
				return $lkw
			}
		}
	}

	return {}
}


Prefops/Newscast public acceptable { cost thresh } {
	if { $cost <= $thresh } {
		return 1
	} else {
		return 0
	}
}




#
# Return the percentage of receivers interested in cat,kw pair
#
Prefops/Newscast public get_wt { ck } {
	$self instvar pref_

	set total_cnt 0
	set interest_cnt 0
	foreach recv_id [array names pref_] {
		incr total_cnt
		if [$self is_interested $recv_id $ck] {
			incr interest_cnt
		}
	}
	return [expr double($interest_cnt) / double($total_cnt)]	
}

#
# Return a list of normalized weights receivers puts on source.
#
Prefops/Newscast public get_wts { } {
	$self instvar cat_arr_

	# not yet clustered
	if ![info exists cat_arr_] {
		return {}
	}
	
	set sum 0
	foreach cat [array names cat_arr_] {
		foreach kw $cat_arr_($cat) {
			set ck [list $cat $kw]
			set ans($ck) [$self get_wt $ck]
			set sum [expr $sum + $ans($ck)]
		}
	}
	foreach cat [array names cat_arr_] {
		foreach kw $cat_arr_($cat) {
			set ck [list $cat $kw]
			set ans($ck) [expr double($ans($ck)) / double($sum)]
		}
	}
	return [array get ans]
}

#
# Receiver or source (host) has died.
# Aged out the associated records to avoid considering them in alg.
#
Prefops/Newscast public delete { ctype id } {
}
	
#
# Heartbeat message from source.
# Keep track of symbols provided by this source.
# lpv: {sym1 sym2 ... symn}
#
Prefops/Newscast public set_df { src_id lpv } {
	$self instvar dataflow_

	set dataflow_($src_id) $lpv

	# insert into partition trigger by receiver pref report
}

#
# Insert preference vector list lpv of receiver recv_id into database.
# lpv: {{category1 {kw1 kw2 ...}} {category2 {kw1 kw2 ...}} ...}
#
Prefops/Newscast public set_pv { recv_id lpv } {
	$self instvar alg_ pref_ cat_arr_

	# reconfigure only if pref has changed since last report
	if { [info exists pref_(recv_id)] && $pref_(recv_id) == $lpv } {
		return
	}

	# first report ever
	if ![info exists cat_arr_] {
		set new $lpv
		foreach p $lpv {
			set cat [lindex $p 0]
			set lkw [lindex $p 1]
			set cat_arr_($cat) $lkw
		}
	} else {
		set new ""
		foreach p $lpv {
			set cat [lindex $p 0]
			set lkw [lindex $p 1]

			# add the new categories and keywords						
			if ![info exists cat_arr_($cat)] {
				lappend new $p
				set cat_arr_($cat) $lkw
			} else {
				set new_lkw [ldiff $lkw $cat_arr_($cat)]
				if { $new_lkw != "" } {
					lappend new [list $cat $new_lkw]
					set cat_arr_($cat) [lunion $cat_arr_($cat) $lkw]
					puts stderr "  cat_arr_($cat) $cat_arr_($cat)"
				}
			}

			# check whether others are still interested in deleted 
			set old [$self get_kw $recv_id $cat]
			set deleted [ldiff $old $lkw]
			foreach kw $deleted {
				set out 1
				foreach id [array names pref_] {
					if { "$id" != "$recv_id" && [lsearch -exact [$self get_kw $id $cat] $kw] != -1 } {
						set out 0
						break
					}
				}
				if { $out == 1 } {
					set cat_arr_($cat) [lremove $cat_arr_($cat) $kw]
					$alg_ delete_src [list $cat $kw]
				}
				
			}
		}
	}

	# insert new src into alg
	foreach p $new {
		set cat [lindex $p 0]
		set lkw [lindex $p 1]
		foreach kw $lkw {
			$alg_ insert_src [list $cat $kw]
		}
	}
	set pref_($recv_id) $lpv		
}

#
# Find the preference vector of receiver recv_id on cat,keyword pair
# pv: {sym [0|1]
#
Prefops/Newscast public get_pv { recv_id ck } {
	$self instvar pref_

	foreach p $pref_($recv_id) {
		set cat [lindex $p 0]
		set lkw [lindex $p 1]
		if { "$cat" == "[lindex $ck 0]" && \
			( [llength [lintersect $lkw [lindex $ck 1]]] > 0 || \
			"$lkw" == "*" ) } {
			return [list $ck 1]
		}
	}
	return [list $ck 0]
}

#
# Get the pv on source.
#
Prefops/Newscast public get_src_pv { ck } {
	$self instvar pref

	set pv ""
	foreach recv_id [array names pref_] {
		lappend pv [list $recv_id [lindex [$self get_pv $recv_id $ck] 1]]
	}
	return $pv
}

#
# Get a zero source pv. 
#
Prefops/Newscast public get_zero_src_pv { } {
	set pv ""
	foreach recv_id [array names pref_] {
		lappend pv [list $recv_id 0]
	}
	return $pv	
}

#
# Add two pv and return a pv.
#
Prefops/Newscast public add { pv1 pv2 } {

	foreach v1 $pv1 {
		set idx1 [lindex $v1 0]
		set val1 [lindex $v1 1]
		foreach v2 $pv2 {
			set idx2 [lindex $v2 0]
			set val2 [lindex $v2 1]			
			if { "$idx1" == "$idx2" } {
				set val [expr $val1 + $val2]
				lappend pv [list $idx1 $val]
				break
			}
		}
	}
	return $pv
}

#
# Subtract pv2 from pv1 and return a pv.
#
Prefops/Newscast public subtract { pv1 pv2 } {
	foreach v1 $pv1 {
		set idx1 [lindex $v1 0]
		set val1 [lindex $v1 1]
		foreach v2 $pv2 {
			set idx2 [lindex $v2 0]
			set val2 [lindex $v2 1]			
			if { "$idx1" == "$idx2" } {
				set val [expr $val1 - $val2]
				lappend pv [list $idx1 $val]
				break
			}
		}
	}
	return $pv
}

#
# Subtract pv2 from pv1 and return a pv.
#
Prefops/Newscast public abs { pv } {
	foreach v1 $pv1 {
		set idx1 [lindex $v1 0]
		set val1 [lindex $v1 1]
		lappend pv [list $idx [expr abs($val1)]]
	}
	return $pv
}


#
# Subtract pv2 from pv1 and return a pv.
#
Prefops/Newscast public scale { pv s } {
	foreach v1 $pv1 {
		set idx1 [lindex $v1 0]
		set val1 [lindex $v1 1]
		lappend pv [list $idx [expr $val1 * $s]]
	}
	return $pv
}

#
# Calculate euclidean distance between two pv.
# Return a real number.
#
Prefops/Newscast public euclidean_dist { pv1 pv2 } {
	set sum 0
	foreach v1 $pv1 {
		set idx1 [lindex $v1 0]
		set val1 [lindex $v1 1]
		foreach v2 $pv2 {
			set idx2 [lindex $v2 0]
			set val2 [lindex $v2 1]			
			if { "$idx1" == "$idx2" } {
				set sum [expr $sum + ($val1 - $val2) * ($val1 - $val2)]
				break
			}
		}
	}
	set dist [expr sqrt($sum)]
	return $dist
}


#
# Return all the sources 
#
Prefops/Newscast public get_srclist { } {
	$self instvar cat_arr_

	foreach cat [array names cat_arr_] {
		foreach kw $cat_arr_($cat) {
			lappend all [list $cat $kw]
		}
	}
	return $all
}

#
# Return all the receivers
#
Prefops/Newscast public get_recvlist { } {
	$self instvar pref_
	
	return [array names pref_]
}
